#include <Application.h>
#include <stdio.h>
#include <be/media/ParameterWeb.h>
#include <be/media/TimeSource.h>
#include <be/media/MediaRoster.h>
#include <be/media/MediaTheme.h>
#include <be/app/Application.h>
#include <be/interface/MultiChannelControl.h>
#include <math.h>
#include <stdio.h>

// Declaration of SystemVolumeApp
class SystemVolumeApp:public BApplication 
{
public:
	SystemVolumeApp(float gain);
	~SystemVolumeApp();
private:
};

// Definition of SystemVolumeApp
SystemVolumeApp::SystemVolumeApp(float gain):BApplication("application/x-vnd.cvs-systemvolumeapp")
{
	media_node mixerNode;
	
	BParameterWeb *mParamWeb;
	BContinuousParameter *mGainParam;	

	BMediaRoster* roster = BMediaRoster::Roster();

	roster->GetAudioMixer(&mixerNode);
	roster->GetParameterWebFor(mixerNode, &mParamWeb);

	// find the master gain parameter
	int32 numParams = mParamWeb->CountParameters();
	BParameter* param = NULL;
	for (int32 i = 0; i < numParams; i++)
	{
		BParameter* p = mParamWeb->ParameterAt(i);
		if (!strcmp(p->Kind(), B_MASTER_GAIN))
		{
			param = p;
			break;
		}
	}

	// we need it to be the right type; this will terminate the app
	// if we didn't find it
	if (param->Type() == BParameter::B_CONTINUOUS_PARAMETER)
	{
		mGainParam = dynamic_cast<BContinuousParameter*>(param);
	}
	else
	{
		printf("ERROR: expected the system mixer to be a BContinuousParameter!\n");
		exit(1);
	}

/*
	// get the single control for this parameter
	BMediaTheme* theme = BMediaTheme::PreferredTheme();
	BControl* control = theme->MakeControlFor(mGainParam);
	mGainControl = dynamic_cast<BMultiChannelControl*>(control);

	// configure the control's "change" message so we can detect it
	mGainControl->SetMessage(new BMessage(gControlMessage));

	// calculate the scaling from the control's range to the parameter's
*/
	float paramMin = mGainParam->MinValue();
	float paramMax = mGainParam->MaxValue();

	float mScale = (paramMax - paramMin) / (1.0 - 0.0);
	
/*
	int32 controlMin, controlMax;
	mGainControl->GetLimitsFor(0, &controlMin, &controlMax);
	mScale = (paramMax - paramMin) / (controlMax - controlMin);
	mOffset = paramMax - mScale * controlMax;
	mControlMin = controlMin;
*/

	// transform the control values into their proper ranges - this is always
	// a linear mapping from the control's range to the parameter's

	float chan1,chan2;
	chan1 = chan2 = gain;

	float chanData[2];
	chanData[0] = mScale * chan1 + paramMin;
	chanData[1] = mScale * chan2 + paramMin;

	// figure out what "now" is in the performance time base associated
	// with the node that the BParameterWeb controls
	BTimeSource* timeSource = roster->MakeTimeSourceFor(mParamWeb->Node());
	bigtime_t now = timeSource->Now();

	// now set the BParameter!
	mGainParam->SetValue(&chanData, sizeof(chanData), now);

	// must delete the time source from BMediaRoster::MakeTimeSourceFor(...)
	timeSource->Release();


	be_app->PostMessage(B_QUIT_REQUESTED);
	return;
}

SystemVolumeApp::~SystemVolumeApp()
{
}

// Main function
int main(int argc, char *argv[])
{
	// I'll skip the parametercheck and just a test-thingy..

	SystemVolumeApp my_app(0.8);
	my_app.Run();

	return(0);
}